@@ -8,7 +8,9 @@ import json
 import importlib
 import iris
 
-from inspect import signature
+from functools import wraps
+
+from inspect import signature, getsource
 
 from dacite import from_dict, Config
 
@@ -21,6 +23,7 @@ class _BusinessHost(_Common):
     """
 
     buffer:int = 64000
+    DISPATCH = []
 
     def input_serialzer(fonction):
         """
@@ -40,6 +43,32 @@ class _BusinessHost(_Common):
                 param2[key] = self._dispatch_serializer(value)
             return fonction(self,*serialized, **param2)
         return dispatch_serializer
+    
+    def input_serialzer_param(position:int,name:str):
+        """
+        It takes a function as an argument, and returns a function that takes the same arguments as the
+        original function, but serializes the arguments before passing them to the original function
+        
+        :param fonction: the function that will be decorated
+        :return: The function dispatch_serializer is being returned.
+        """
+        def input_serialzer_param(fonction):
+            @wraps(fonction)
+            def dispatch_serializer(self,*params, **param2):
+                # Handle positional arguments
+                serialized=[]
+                for i,param in enumerate(params):
+                    if i == position:
+                        serialized.append(self._dispatch_serializer(param))
+                    else:
+                        serialized.append(param)
+                # Handle keyword arguments
+                for key, value in param2.items():
+                    if key == name:
+                        param2[key] = self._dispatch_serializer(value)
+                return fonction(self,*serialized, **param2)
+            return dispatch_serializer
+        return input_serialzer_param
 
     def output_deserialzer(fonction):
         """
@@ -67,10 +96,10 @@ class _BusinessHost(_Common):
             # Handle positional arguments
             serialized=[]
             for param in params:
-                serialized.append(self._dispatch_serializer(param))
+                serialized.append(self._dispatch_deserializer(param))
             # Handle keyword arguments
             for key, value in param2.items():
-                param2[key] = self._dispatch_serializer(value)
+                param2[key] = self._dispatch_deserializer(value)
             return fonction(self,*serialized, **param2)
         return dispatch_deserializer
 
@@ -87,7 +116,7 @@ class _BusinessHost(_Common):
             return self._dispatch_serializer(fonction(self,*params, **param2))
         return dispatch_serializer
 
-    @input_serialzer
+    @input_serialzer_param(1,'request')
     @output_deserialzer
     def send_request_sync(self, target, request, timeout=-1, description=None):
         """ Send the specified message to the target business process or business operation synchronously.
@@ -107,7 +136,7 @@ class _BusinessHost(_Common):
 
         return self.iris_handle.dispatchSendRequestSync(target,request,timeout,description)
 
-    @input_serialzer
+    @input_serialzer_param(1,'request')
     def send_request_async(self, target, request, description=None):
         """ Send the specified message to the target business process or business operation asynchronously.
         Parameters:
@@ -162,8 +191,8 @@ class _BusinessHost(_Common):
             return message
         else:
             # todo : decorator takes care of all the parameters, so this should never happen
-            return message
-            #raise TypeError("The message must be an instance of a class that is a subclass of Message or IRISObject %Persistent class.")
+            # return message
+            raise TypeError("The message must be an instance of a class that is a subclass of Message or IRISObject %Persistent class.")
 
     def _serialize_message(self,message):
         """ Converts a python dataclass message into an iris grongier.pex.message.
@@ -352,6 +381,59 @@ class _BusinessHost(_Common):
         Name of the registred Adapter
         """
         return 
+    
+    def on_get_connections(self) -> list:
+        """
+        The OnGetConnections() method returns all of the targets of any SendRequestSync or SendRequestAsync
+        calls for the class. Implement this method to allow connections between components to show up in 
+        the interoperability UI.
+
+        Returns:
+            An IRISList containing all targets for this class. Default is None.
+        """
+        ## Parse the class code to find all invocations of send_request_sync and send_request_async
+        ## and return the targets
+        targer_list = []
+        # get the source code of the class
+        source = getsource(self.__class__)
+        # find all invocations of send_request_sync and send_request_async
+        for method in ['send_request_sync','send_request_async','SendRequestSync','SendRequestAsync']:
+            i = source.find(method)
+            while i != -1:
+                j = source.find("(",i)
+                if j != -1:
+                    k = source.find(",",j)
+                    if k != -1:
+                        target = source[j+1:k]
+                        if target.find("=") != -1:
+                            # it's a keyword argument, remove the keyword
+                            target = target[target.find("=")+1:].strip()
+                        if target not in targer_list:
+                            targer_list.append(target)
+                i = source.find(method,i+1)
+
+        for target in targer_list:
+            # if target is a string, remove the quotes
+            if target[0] == "'" and target[-1] == "'":
+                targer_list[targer_list.index(target)] = target[1:-1]
+            elif target[0] == '"' and target[-1] == '"':
+                targer_list[targer_list.index(target)] = target[1:-1]
+            # if target is a variable, try to find the value of the variable
+            else:
+                self.on_init()
+                try:
+                    if target.find("self.") != -1:
+                        # it's a class variable
+                        targer_list[targer_list.index(target)] = getattr(self,target[target.find(".")+1:])
+                    elif target.find(".") != -1:
+                        # it's a class variable
+                        targer_list[targer_list.index(target)] = getattr(getattr(self,target[:target.find(".")]),target[target.find(".")+1:])
+                    else:
+                        targer_list[targer_list.index(target)] = getattr(self,target)
+                except Exception as e:
+                    pass
+
+        return targer_list
 
 # It's a subclass of the standard JSONEncoder class that knows how to encode date/time, decimal types,
 # and UUIDs.
